/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>
static Widget texts[4];

/* (1) Navigate to next input area */
static void MoveFocusCB(w, client_data, call_data)
    Widget w;
    XtPointer client_data;  /* text number is passed */
    XtPointer call_data;
{
    /* If the text number is 0, 1 or 2, move the focus to next text */
    if ((int)client_data < 3)
        XmProcessTraversal(w, XmTRAVERSE_NEXT_TAB_GROUP);
    /* Next of the text 3 is text 0 */
    else
        XmProcessTraversal(texts[0], XmTRAVERSE_CURRENT);
}

/* (2) Indicate the error location */
static void CheckInputCB(w, client_data, call_data)
    Widget w;
    XtPointer client_data;
    XtPointer call_data;
{
    int   i;
    char *string[4];

    for (i = 0; i < 4; i++) {
        /* Check if string is entered in the text */
        string[i] = XmTextFieldGetString(texts[i]);
        if (string[i] == NULL || strlen(string[i]) == 0) {
        /* If none is entered, ring a bell and set focus to the text widget */
            XBell(XtDisplay(texts[i]), 0);
            XmProcessTraversal(texts[i], XmTRAVERSE_CURRENT);
            return;
        }
        printf("Entered text in %s: %s\n", XtName(texts[i]), string[i]);
        XtFree(string[i]);  /* Free if not necessray */
    }
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, panel, field, labels[4], regist;
    static char *lnames[] = { "label0", "label1", "label2", "label3" };
    static char *tnames[] = { "text0", "text1", "text2", "text3" };
    Arg    al[20];
    int    ac, i;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Movefocus", NULL, 0,
                               &argc, argv, NULL, al, ac);
    ac = 0;
    panel = XmCreateRowColumn(toplevel, "panel", al, ac);
    XtManageChild(panel);
    ac = 0;
    field = XmCreateRowColumn(panel, "field", al, ac);
    XtManageChild(field);

    for (i = 0; i < 4; i++) {
        ac = 0;
        labels[i] = XmCreateLabel(field, lnames[i], al, ac);
        texts[i] = XmCreateTextField(field, tnames[i], al, ac);
        XtAddCallback(texts[i], XmNactivateCallback, MoveFocusCB, (XtPointer)i);
    }                                                /* Pass the text number */
    XtManageChildren(labels, 4);
    XtManageChildren(texts, 4);

    /* The text 0 should be entered first */
    XtVaSetValues(panel, XmNinitialFocus, field, NULL);
    XtVaSetValues(field, XmNinitialFocus, texts[0], NULL);

    ac = 0;
    regist = XmCreatePushButton(panel, "regist", al, ac);
    XtManageChild(regist);
    XtAddCallback(regist, XmNactivateCallback, CheckInputCB, NULL);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}
